
-- SurgeonGeneral Listen Client by Brimmstone
-- Version 1.0

SGLC = LibStub("AceAddon-3.0"):NewAddon("SurgeonGeneral Listen Client", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceSerializer-3.0")

local LTT=LibStub:GetLibrary("LibQTip-1.0");
local tooltip;
local tooltipVisible;
local HealerName = {};

local options = {
	name = "SurgeonGeneral Listen Client",
	handler = SGLC,
    type = "group",
    args = {
		TooltipScale = {
			type = "range",
			name = "Tooltip Scale",
			desc = "Set scale of mouseover tooltip",
			min = 0.5,
			max = 1.5,
			step = 0.05,
			bigStep = 0.05,
			get = function() return SGLC.db.profile.TooltipScale end,
			set = function(_,value)
					SGLC.db.profile.TooltipScale = value
					tooltip:SetScale(SGLC.db.profile.TooltipScale)
				  end,
		},
	},
}

local defaults = {
	profile = {
		Verbose = false,
		Condense = true,
		TooltipScale = 1.0,
		SwapStyle = 1,
	},
}

function SGLC:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SurgeonGeneralListenClientDB", defaults, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SurgeonGeneral Listen Client", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SurgeonGeneral Listen Client", "SurgeonGeneral Listen Client");
	tooltip = LTT:Acquire("SGLC_TT",3,"LEFT","CENTER","RIGHT");
	
	SGLC:RegisterComm("SGLC");
end

function SGLC:ForcePrint(msg,color)
	if color ~= nil then
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99SGLC: |cff"..color..msg.."|r");
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99SGLC: |cffffffff"..msg.."|r");
	end
end

function SGLC:OnLoad()
	
end

function ClassColor(class)
	if class ~= nil then
		return string.format( "%2x%2x%2x", RAID_CLASS_COLORS[class].r * 255, RAID_CLASS_COLORS[class].g * 255, RAID_CLASS_COLORS[class].b * 255 )
	else
		return "7f7f7f";
	end
end

function ColorText(text,color)
	if color == nil then
		return text;
	else
		return "|cff"..color..text.."|r";
	end
end

function SGLC:DissectMessage(Success,BossName,Phases,...)
	--[[
	if Success then
		SGLC:ForcePrint("Deserialize Success");
	else
		SGLC:ForcePrint("Deserialize Failure");
	end
	]]--
	
	Phases = tonumber(Phases);
	local NumColumns = Phases + 2;
	local x,y;
	local t = {};
	
	tooltipVisible = tooltip:IsShown();
	
	tooltip:Hide();
	tooltip:Clear();
	
	LTT:Release(tooltip);
	tooltip = LTT:Acquire("SGLC_TT",3,"LEFT","CENTER","RIGHT");
	tooltip:SetScale(self.db.profile.TooltipScale);
	
	for x = 1,Phases do
		table.insert(t,"CENTER");
	end
	
	tooltip:SetColumnLayout(NumColumns,"LEFT","LEFT",unpack(t));
	tooltip:AddHeader("a");
	tooltip:SetCell(1,1,BossName,"CENTER",NumColumns);
	tooltip:AddLine(" ");
	
	if(Phases > 1) then
		wipe(t)
		table.insert(t," ");
		table.insert(t," ");
		for x = 1,Phases do
			local PhaseName = select(x+10,...);
			table.insert(t,PhaseName);
		end
		tooltip:AddLine(unpack(t));
	end
	
	wipe(HealerName);
	for x = 1,10 do
		HealerName[x] = select(x,...);
		if HealerName[x] ~= "" then
			wipe(t);
			local _,EnglishClass = UnitClass(HealerName[x]);
			local UnitTexture;
			
			if UnitIsDeadOrGhost(HealerName[x]) then
				UnitTexture = "|TInterface\\Addons\\SurgeonGeneral\\Icons\\Tombstone:0|t";
			elseif UnitExists(HealerName[x]) and UnitInRaid(HealerName[x]) and (not UnitIsConnected(HealerName[x])) then
				UnitTexture = "|TInterface\\GossipFrame\\ActiveQuestIcon:0|t";
			else
				UnitTexture = "";
			end
			
			local LineColor = ClassColor(EnglishClass);
			
			table.insert(t,UnitTexture);
			table.insert(t,ColorText(HealerName[x],LineColor));
			for y = 1,Phases do
				local temp = select(x*5+10+y,...);
				table.insert(t,ColorText(temp,LineColor));
			end
			tooltip:AddLine(unpack(t));
		end
	end
	
	
end

function SGLC:OnCommReceived(prefix, message, distro, sender)
	--SGLC:ForcePrint("OnCommRx: "..prefix..", "..message..", "..distro..", "..sender);
	SGLC:DissectMessage(SGLC:Deserialize(message));
	
	local Hour,Minute = GetGameTime();
	local ServerTime;
	if Hour > 12 then
		ServerTime = string.format("%u:%02u pm",(Hour-12),Minute);
	elseif Hour == 0 then
		ServerTime = string.format("%u:%02u am",12,Minute);
	else
		ServerTime = string.format("%u:%02u am",Hour,Minute);
	end
	
	tooltip:AddLine(" ");
	tooltip:AddLine("."); -- For Sender Info
	tooltip:AddLine("."); -- For Time Info
	local NumLines = tooltip:GetLineCount();
	local NumColumns = tooltip:GetColumnCount();
	tooltip:SetCell(NumLines-1,1,"from: "..sender,"RIGHT",NumColumns);
	tooltip:SetCell(NumLines-0,1,ServerTime,"RIGHT",NumColumns);
	
	if tooltipVisible then
		tooltip:SmartAnchorTo( self )
		tooltip:Show();
	end
end


function SGLC:UpdateHealerStatus()
	local NumColumns = tooltip:GetColumnCount();
	local CurrentLine,UnitTexture;
	local x;
	
	if NumColumns > 3 then
		CurrentLine = 4;
	elseif NumColumns == 3 then
		CurrentLine = 3;
	else
		return;
	end
	
	for x = 1,10 do
		if (HealerName[x] ~= "") and (HealerName[x] ~= nil) then
			if UnitIsDeadOrGhost(HealerName[x]) then
				UnitTexture = "|TInterface\\Addons\\SurgeonGeneral\\Icons\\Tombstone:0|t";
			elseif UnitExists(HealerName[x]) and UnitInRaid(HealerName[x]) and (not UnitIsConnected(HealerName[x])) then
				UnitTexture = "|TInterface\\GossipFrame\\ActiveQuestIcon:0|t";
			else
				UnitTexture = "";
			end
			
			tooltip:SetCell(CurrentLine,1,UnitTexture,"CENTER",1);
			CurrentLine = CurrentLine + 1;
		end
	end
end




local LDB = LibStub("LibDataBroker-1.1", true)
local L_SG_LEFT = "Left"
local L_SG_RIGHT = "Right"
	local launcher = LDB:NewDataObject("SGLC", {
	type = "data source",
	text = "SGLC",
	OnClick = function(_, msg)
		if msg == "LeftButton" then
			--
		elseif msg == "RightButton" then
			--
		end
	end,
	icon = "Interface\\Icons\\Spell_ChargePositive",
	OnEnter = function(self)
		SGLC:UpdateHealerStatus();
		tooltip:SmartAnchorTo( self )
		tooltip:Show();
	end,
	OnLeave = function(self)
		tooltip:Hide();
	end,
})